#######################################################
# A coarse MILP model of distributed generation (DG)  #
# AMPL coding by                                      #
# Fu Lin                                              #
# Argonne National Laboratory, 2014                   #
#######################################################

### SETS ###
set Time;						# ... time intervals
set Periods;                     			# ... set of periods
set Techs  						# ... set of technologies
    := { "PowerSOFC", "ChpSOFC", "Storage", "Boiler", "Battery"}; 
set TechsSOFC := { "PowerSOFC", "ChpSOFC" };            # ... SOFC technologies

### PARAMETERS ###
param CapCost{Techs};           			# ... capital cost of technologies
param MaxPowerRate{Techs};         			# ... maximum power output rating of technologies
param MinPowerRate{Techs};         			# ... minimum power output rating of technologies
param UpTech{Techs};		  			# ... upper bound on each technology
param OpeMainCost{Techs};    				# ... operation and maintenance cost  
param PriceGas{Time};					# ... price of natural gas
param PriceElectr{Time};				# ... price of electricity
param PeakPrice{Periods};				# ... peak demand price of power from utility in periods
param TherEff{Techs};					# ... thermal efficiency of each technology
param PowerDemand{Time};				# ... average power demand
param HeatDemand{Time};					# ... average heat demand
param RateElectrEff{TechsSOFC}; 			# ... rated eletric efficiency
param AveTherEff{Techs};				# ... average thermal efficiency
param HeatLoss;                				# ... average ambient heat loss of water stored in storage
param PowerLoss;                			# ... average power loss of in battery
param TherCap;						# ... thermal storage capacity
param BatteryCap;					# ... battery storage capacity
param BoilerCap;					# ... thermal capacity of boiler
param EmiCarbPower;					# ... carbon emissions rate for utility power
param EmiCarbGas;					# ... carbon emissions rate for gas combustion

param SwitchCost{TechsSOFC};				# ... cost for switching on or off technologies

param StartPeriod{Periods};				# ... Starting hour of each period
param EndPeriod{Periods};				# ... Ending hour of each period

param NumHour := 24;                                    # ... number of hours in a day

param DiscFactor := 0.03/8760;				# ... hourly discount factor (delta)
param Beta := 1 - DiscFactor;				# ... beta^t is the value at current time
param DiscFactorDay := 0.03/8760*NumHour;		# ... daily discount factor (delta)
param BetaDay := 1 - DiscFactorDay;			# ... BetaDay^d is the value at current time
param LifeExp := 87600;					# ... life-expectancy of tech = number of hours in 10 years

### derived index sets ###
set Units{j in TechsSOFC} := 1..UpTech[j] ordered;      # ... set of units of type j

################# SETS, PARAMETERS, VARIABLES for coarse and semi-coarse models ####################

set Days = {1..card(Time)/NumHour};		# ... set of days in the time horizon
set OnOffCol{TechsSOFC} ordered; 		# ... set of the number of on-unit profiles
set GenCol{j in TechsSOFC, OnOffCol[j]} ordered;		
                                                # ... set of generation profiles associated with operation profiles
set PurCol;				        # ... set of purchase profiles
set BattCol;   				        # ... set of battery profiles
set BattIOCol;				        # ... set of battery Input/Output profiles
set HeatStorCol;			        # ... set of heat storage profiles
set HeatStorOutCol;                             # ... set of heat output profiles from storage 
set HeatBoilOutCol;                             # ... set of heat output profiles from boiler
set HeatStorInCol;                              # ... set of heat input to storage profiles

### Columns/Profiles ###
param OnOffProf{1..NumHour, j in TechsSOFC, OnOffCol[j]};		
						# ... On/off profiles
param GenProf{1..NumHour,j in TechsSOFC, k in OnOffCol[j],i in GenCol[j,k]};			
						# ... generation profiles
param PurProf{1..NumHour,PurCol};		# ... purchased profiles
param PurInfProf{1..24} default 1;		

param BattProf{1..NumHour,BattCol};		# ... battery profiles
param BattIOProf{1..NumHour,BattIOCol};		# ... battery Input/Output profiles
param HeatStorProf{1..NumHour,HeatStorCol};	# ... heat storage profiles
param HeatStorOutProf{1..NumHour,HeatStorOutCol};	
                                                # ... heat output profiles from storage 
param HeatBoilOutProf{1..NumHour,HeatBoilOutCol};	
                                                # ... heat output profiles from boiler
param HeatInfProf{1..24} default 1;		# ... infinite heat output profile from boiler

param HeatStorInProf{1..NumHour,HeatStorInCol};      
                                                # ... heat storage input profiles
param SwitProf{t in 1..(NumHour-1), j in TechsSOFC, k in OnOffCol[j]} = abs(OnOffProf[t+1,j,k] - OnOffProf[t,j,k]);		                         		           # ... switching profiles
param LifeExpDay := LifeExp/NumHour;	     	# ... life-expectancy of tech = number of hours in 10 years	

param TherEffElectChpSOFC := TherEff["ChpSOFC"]/RateElectrEff["ChpSOFC"] ;

### VARIABLES (start with lower case letter) ###
var numTech{j in Techs} integer, >= 0, <= UpTech[j];	
						# ... number of each technology acquired
### Coefficients for profiles ###
var onOffCoeff{Days, j in TechsSOFC, i in Units[j], k in OnOffCol[j]} binary;
var genCoeff{Days, j in TechsSOFC, i in Units[j], k in OnOffCol[j], l in GenCol[j,k]} >=0, <=1;
var purCoeff{Days,PurCol} >=0, <=1;
var battCoeff{Days,BattCol} >=0, <=1;
var heatStorCoeff{Days,HeatStorCol} >=0, <=1;
					        # ... heat storage input coefficient is set to be the same as the heat storage coefficient
var heatBoilOutCoeff{Days,HeatBoilOutCol} >=0, <=1;
var maxPower{Periods} >= 0;	  	        # ... maximum power purchased from utility in periods

var purInfCoeff{Days} >= 0;                     # ... non-negative coefficient for the special purchased profile
var heatInfCoeff{Days} >= 0;                    # ... non-negative coefficient for the special heat generation profile

### OBJECTIVE ###
# ... minimize cost of acquisition and operation

minimize TotalCost: sum{j in Techs} CapCost[j] * numTech[j]     
                                                # ... 1st-stage cost

	 	    + LifeExpDay/card(Days) * 

		    (sum{d in Days, j in TechsSOFC, i in Units[j], k in OnOffCol[j], l in GenCol[j,k], h in 1..NumHour} 

                     (OpeMainCost[j] + (PriceGas[(d-1)*NumHour+h]+EmiCarbGas)/RateElectrEff[j]) 
					* genCoeff[d,j,i,k,l] * GenProf[h,j,k,l]  * Beta^(NumHour*(d-1)+h)                        
						# ... maintenance cost and gas cost for SOFCs

		    + sum{d in Days, j in TechsSOFC, i in Units[j], k in OnOffCol[j], h in 1..(NumHour-1)} 
			SwitchCost[j] * onOffCoeff[d,j,i,k] * SwitProf[h,j,k] * Beta^(NumHour*(d-1)+h)     # ... switching cost

 		    + sum{d in Days, k in PurCol, h in 1..NumHour} 
			(PriceElectr[(d-1)*NumHour+h] + EmiCarbPower) * purCoeff[d,k] * PurProf[h,k] * Beta^(NumHour*(d-1)+h)                					   # ... cost for purchased power from utility
	
 		    + sum{d in Days, h in 1..24} (PriceElectr[(d-1)*24+h] + EmiCarbPower) * purInfCoeff[d] * PurInfProf[h] * Beta^(24*(d-1)+h)                       		   # ... cost for special purchased power from utility		  

                    + sum{d in Days, k in HeatBoilOutCol, h in 1..NumHour} 
			(PriceGas[(d-1)*NumHour+h] + EmiCarbGas)/AveTherEff["Boiler"] * heatBoilOutCoeff[d,k] 
							* HeatBoilOutProf[h,k] * Beta^(NumHour*(d-1)+h)                 
                                                # ... gas cost for boiler

		    + sum{d in Days, h in 1..24} (PriceGas[(d-1)*24+h] + EmiCarbGas)/AveTherEff["Boiler"] 
				* heatInfCoeff[d]*HeatInfProf[h] * Beta^(24*(d-1)+h)
                                                # ... gas cost for special generation profile for boiler

		    + sum{p in Periods} min( card(Days)/30 , 1.0) * PeakPrice[p] * maxPower[p] * Beta^EndPeriod[p]);         
					        # ... monthly (=period) premium


### CONSTRAINTS ###
subject to

        # =================================  POWER  ==============================
	# ... power demand
	PowerDemandConstr{d in Days}:
          	sum{j in TechsSOFC, i in Units[j], k in OnOffCol[j], l in GenCol[j,k], h in 1..NumHour} 
			genCoeff[d,j,i,k,l]*GenProf[h,j,k,l] 
              + sum{k in PurCol, h in 1..NumHour} purCoeff[d,k]*PurProf[h,k] 
	      + sum{h in 1..NumHour} purInfCoeff[d]*PurInfProf[h]			# ... special purchased profile
              - sum{k in BattIOCol, h in 1..NumHour} battCoeff[d,k]*BattIOProf[h,k] 
	      >= sum{h in 1..NumHour} PowerDemand[NumHour*(d-1)+h];

	# ... hourly power demand
	HourlyPowerDemandConstr{d in Days, h in 1..NumHour}:
          	sum{j in TechsSOFC, i in Units[j], k in OnOffCol[j], l in GenCol[j,k]} 
			genCoeff[d,j,i,k,l]*GenProf[h,j,k,l] 
              + sum{k in PurCol} purCoeff[d,k]*PurProf[h,k] 
	      +                  purInfCoeff[d]*PurInfProf[h]			# ... special purchased profile
              - sum{k in BattIOCol} battCoeff[d,k]*BattIOProf[h,k] - PowerDemand[NumHour*(d-1)+h] >=0;

	# ... special ordered set constraints for operation profiles
        SOS1OnUnitCoeff{d in Days, j in TechsSOFC, i in Units[j]}:
	        sum{k in OnOffCol[j]} onOffCoeff[d,j,i,k] <= 1;


	# ... number of on units bounded by the number of units acquired
	OnUnitsBound{d in Days, j in TechsSOFC}:
		sum{i in Units[j], k in OnOffCol[j], h in 1..NumHour} onOffCoeff[d,j,i,k] * OnOffProf[h,j,k] <= NumHour*numTech[j];
	
	# ... hourly number of on units bounded by the number of units acquired
	HourlyOnUnitsBound{d in Days, h in 1..NumHour, j in TechsSOFC}:
		numTech[j] - sum{i in Units[j], k in OnOffCol[j]} onOffCoeff[d,j,i,k] * OnOffProf[h,j,k] >=0;	

	# ... break symmetries between identical units
	SymmBreak{d in Days, j in TechsSOFC, i in Units[j] diff {last(Units[j])}}:
		sum{k in OnOffCol[j], h in 1..NumHour} onOffCoeff[d,j,i,k]*OnOffProf[h,j,k] 
		>= sum{k in OnOffCol[j], h in 1..NumHour} onOffCoeff[d,j,i+1,k]*OnOffProf[h,j,k];

	HourlySymmBreak{d in Days, h in 1..24, j in TechsSOFC, i in Units[j] diff {last(Units[j])}}:
		sum{k in OnOffCol[j]} onOffCoeff[d,j,i,k]*OnOffProf[h,j,k] 
		- sum{k in OnOffCol[j]} onOffCoeff[d,j,i+1,k]*OnOffProf[h,j,k] >=0;
	
	# ... convex combination of generation profiles when the corresponding operation profile is selected
        CvxCombGenCoeff{d in Days, j in TechsSOFC, i in Units[j], k in OnOffCol[j]}:
		sum{l in GenCol[j,k]} genCoeff[d,j,i,k,l] <= onOffCoeff[d,j,i,k];

	# ... maximum utility 
	MaxUtility{p in Periods, d in floor((StartPeriod[p]-1)/NumHour+1)..floor(EndPeriod[p]/NumHour)}: 
		maxPower[p]*NumHour - sum{k in PurCol, h in 1..NumHour} purCoeff[d,k]*PurProf[h,k]
				    - sum{h in 1..NumHour} purInfCoeff[d] * PurInfProf[h] >=0;
      
	# ... hourly maximum utility
	HourlyMaxUtility{p in Periods, t in StartPeriod[p]..EndPeriod[p]}: 
		maxPower[p] - sum{k in PurCol} purCoeff[floor((t-1)/24)+1,k]*PurProf[((t-1) mod 24)+1,k]
			    - purInfCoeff[floor((t-1)/24)+1] * PurInfProf[((t-1) mod 24)+1] >=0;

	# ... power output from battery is upper bounded by battery capacity	
	PowerStorageBound{d in Days}:
		sum{k in BattCol, h in 1..NumHour} battCoeff[d,k]*BattProf[h,k] <= BatteryCap * NumHour * numTech["Battery"];
	
	# ... hourly power output from battery is upper bounded by battery capacity
	HourlyPowerStorageBound{d in Days, h in 1..NumHour}:
		 BatteryCap * numTech["Battery"] - sum{k in BattCol} battCoeff[d,k]*BattProf[h,k] >=0;

	# ... energy storage
	EnergyStorageEnd{d in 1..card(Days)-1}:
              	sum{k in BattCol} battCoeff[d+1,k]*BattProf[1,k] 
	        = (1-PowerLoss)*sum{k in BattCol} battCoeff[d,k]*BattProf[NumHour,k] 
		+ sum{k in BattIOCol} battCoeff[d,k]*BattIOProf[NumHour,k]; 

	# ... boundary condition for power storage in battery
	BatteryBoundaryCond:
		sum{k in BattCol} battCoeff[1,k]*BattProf[1,k] = sum{k in BattCol} battCoeff[card(Days),k]*BattProf[NumHour,k];	
           
        # ... initial battery storage
	InitBattery:
		sum{k in BattCol} battCoeff[1,k]*BattProf[1,k] = 0;
	
        # =================================  HEAT ==============================

	# ... heat demand
	# note that heatStorOutCoeff is set to be the same as heatStorCoeff
	HeatDemandConstr{d in Days}:
		sum{k in HeatStorOutCol, h in 1..NumHour} heatStorCoeff[d,k] * HeatStorOutProf[h,k] 
              + sum{k in HeatBoilOutCol, h in 1..NumHour} heatBoilOutCoeff[d,k] * HeatBoilOutProf[h,k] 
	      + sum{h in 1..NumHour} heatInfCoeff[d]*HeatInfProf[h]		# ... special heat generation profile
              - sum{h in 1..NumHour} HeatDemand[NumHour*(d-1)+h] >=0;
	
	# ... hourly heat demand
	# ... note that heatStorOutCoeff is set to be the same as heatStorCoeff
	HourlyHeatDemandConstr{d in Days, h in 1..NumHour}:
		sum{k in HeatStorOutCol} heatStorCoeff[d,k] * HeatStorOutProf[h,k] 
              + sum{k in HeatBoilOutCol} heatBoilOutCoeff[d,k] * HeatBoilOutProf[h,k] 
	      + heatInfCoeff[d]*HeatInfProf[h]			# ... special heat generation profile
	      - HeatDemand[NumHour*(d-1)+h] >=0;

	BoundaryCond:
		sum{k in HeatStorCol} heatStorCoeff[1,k] * HeatStorProf[1,k] = sum{k in HeatStorCol} heatStorCoeff[card(Days),k] * HeatStorProf[NumHour,k];

        InitHeatStorage:
                sum{k in HeatStorCol} heatStorCoeff[1,k] * HeatStorProf[1,k] = 0;

	# ... heat input to storage from ChpSOFC
        # ... note that heat input storage coefficient is set to be the same as the heat storage coefficient
	HeatInBound{d in Days}:
		sum{k in HeatStorCol, h in 1..(NumHour-1)} heatStorCoeff[d,k]*HeatStorOutProf[h,k] + sum{k in HeatStorCol, h in 1..(NumHour-1)} heatStorCoeff[d,k]*(HeatStorProf[h+1,k] - (1-HeatLoss)*HeatStorProf[h,k]) <= TherEff["ChpSOFC"]/RateElectrEff["ChpSOFC"] * sum{k in OnOffCol["ChpSOFC"], i in Units["ChpSOFC"], l in GenCol["ChpSOFC",k], h in 1..(NumHour-1)} genCoeff[d,"ChpSOFC",i,k,l]*GenProf[h,"ChpSOFC",k,l];
	
	# ... hourly heat input to storage from ChpSOFC
        # ... note that heat input storage coefficient is set to be the same as the heat storage coefficient
	HourlyHeatInBound{d in Days, h in 1..(NumHour-1)}:
		TherEff["ChpSOFC"]/RateElectrEff["ChpSOFC"] * sum{k in OnOffCol["ChpSOFC"], i in Units["ChpSOFC"], l in GenCol["ChpSOFC",k]} genCoeff[d,"ChpSOFC",i,k,l]*GenProf[h,"ChpSOFC",k,l] - (sum{k in HeatStorCol} heatStorCoeff[d,k]*HeatStorOutProf[h,k] + sum{k in HeatStorCol} heatStorCoeff[d,k]*(HeatStorProf[h+1,k] - (1-HeatLoss)*HeatStorProf[h,k])) >=0;

	# ... heat input to storage from ChpSOFC
	HeatInBoundEnd{d in 1..card(Days)-1}:
		sum{k in HeatStorCol} heatStorCoeff[d,k]*HeatStorOutProf[NumHour,k] - sum{k in HeatStorCol} heatStorCoeff[d,k]*(1-PowerLoss)*HeatStorProf[NumHour,k] + sum{k in HeatStorCol} heatStorCoeff[d+1,k]*HeatStorProf[1,k] 
	<= TherEff["ChpSOFC"]/RateElectrEff["ChpSOFC"] * sum{k in OnOffCol["ChpSOFC"], i in Units["ChpSOFC"], l in GenCol["ChpSOFC",k]} genCoeff[d,"ChpSOFC",i,k,l]*GenProf[NumHour,"ChpSOFC",k,l];

	# ... heat output from boiler is upper bounded by the boiler capacity
	HeatOutBoilerBound{d in Days}:
		sum{k in HeatBoilOutCol, h in 1..NumHour} heatBoilOutCoeff[d,k] * HeatBoilOutProf[h,k] 
		 + sum{h in 1..NumHour} heatInfCoeff[d]*HeatInfProf[h]			# ... special heat generation profile
		<= BoilerCap * NumHour * numTech["Boiler"];

	# ... hourly heat output from boiler is upper bounded by the boiler capacity
	HourlyHeatOutBoilerBound{d in Days, h in 1..NumHour}:
		BoilerCap * numTech["Boiler"] 
		- sum{k in HeatBoilOutCol} heatBoilOutCoeff[d,k] * HeatBoilOutProf[h,k]
		- heatInfCoeff[d]*HeatInfProf[h] >=0;	
		# ... special heat generation profile

	# ... heat output from storage is upper bounded by the storage capacity
        # ... note that the coefficient for storage and coefficient for storage output is set to be the same
	HeatStorageBound{d in Days}:
		sum{k in HeatStorCol, h in 1..NumHour} heatStorCoeff[d,k] * HeatStorOutProf[h,k] <= TherCap * NumHour * numTech["Storage"];

	# ... hourly heat output from storage is upper bounded by the storage capacity
        # ... note that the coefficient for storage and coefficient for storage output is set to be the same
	HourlyHeatStorageBound{d in Days, h in 1..NumHour}:
		TherCap * numTech["Storage"] - sum{k in HeatStorCol} heatStorCoeff[d,k] * HeatStorOutProf[h,k] >= 0;

        # ... constraints on number of on-unit profiles and convex combination of profiles
        CvxCombPurCoeff{d in Days}:
		sum{k in PurCol} purCoeff[d,k] <= 1;

        CvxCombBattCoeff{d in Days}:
		sum{k in BattCol} battCoeff[d,k] <= 1;

        CvxCombHeatStorCoeff{d in Days}:
		sum{k in HeatStorCol} heatStorCoeff[d,k] <= 1;

        CvxCombHeatBoilOutCoeff{d in Days}:
		sum{k in HeatBoilOutCol} heatBoilOutCoeff[d,k] <= 1;


