################################################
# A MILP model of distributed generation (DG)  #
# AMPL coding by                               #
# Fu Lin                                       #
# Argonne National Laboratory, 2014            #
################################################

### SETS ###
set Time ordered;					# ... time intervals
set Periods;                     			# ... set of periods
set Techs  						# ... set of technologies
    := { "PowerSOFC", "ChpSOFC", "Storage", "Boiler", "Battery"}; 
set TechsSOFC := { "PowerSOFC", "ChpSOFC" };            # ... SOFC technologies
set TechsHeat := {"Storage", "Boiler"}; 	        # ... set of old technologies

### PARAMETERS ###
param CapCost{Techs};           			# ... capital cost of technologies
param MaxPowerRate{Techs};         			# ... maximum power output rating of technologies
param MinPowerRate{Techs};         			# ... minimum power output rating of technologies
param UpTech{Techs};		  			# ... upper bound on each technology
param OpeMainCost{Techs};    				# ... operation and maintenance cost  
param PriceGas{Time};					# ... price of natural gas
param PriceElectr{Time};				# ... price of electricity
param PeakPrice{Periods};				# ... peak demand price of power from utility in periods
param TherEff{Techs};					# ... thermal efficiency of each technology
param PowerDemand{Time};				# ... average power demand
param HeatDemand{Time};					# ... average heat demand
param RateElectrEff{TechsSOFC}; 			# ... rated eletric efficiency
param AveTherEff{Techs};				# ... average thermal efficiency
param HeatLoss;                				# ... average ambient heat loss of water stored in storage
param PowerLoss;                			# ... average power loss of in battery
param TherCap;						# ... thermal storage capacity
param BatteryCap;					# ... battery storage capacity
param BoilerCap;					# ... thermal capacity of boiler
param EmiCarbPower;					# ... carbon emissions rate for utility power
param EmiCarbGas;					# ... carbon emissions rate for gas combustion

param SwitchCost{TechsSOFC};				# ... cost for switching on or off technologies

param StartPeriod{Periods};				# ... Starting hour of each period
param EndPeriod{Periods};				# ... Ending hour of each period

param DiscFactor := 0.03/8760;				# ... hourly discount factor (delta)
param Beta := 1 - DiscFactor;				# ... beta^t is the value at current time
param LifeExp := 87600;					# ... life-expectancy of tech = number of hours in 10 years

### derived index sets ###
set Units{j in TechsSOFC} := 1..UpTech[j] ordered;      # ... set of units of type j


### VARIABLES (start with lower case letter) ###
var numTech{j in Techs} integer, >= 0, <= UpTech[j];	# ... number of each technology acquired
var power{j in TechsSOFC, Time, i in Units[j]} >=0;	# ... power generated
var powerUtility{Time} >=0;				# ... power purchased from utility in time
var powerInPlus{Time} >=0;                              # ... power input to battery 
var powerInMinus{Time} >=0;                             # ... power output from battery 
var powerStorage{Time} >=0;                             # ... power storage in battery
var maxPower{Periods} >=0;				# ... maximum power purchased from utility in periods

var heatOut{j in TechsHeat, Time} >=0;		 	# ... heat output from technologies
var heatIn{Time} >=0;					# ... heat input to storage
var heatStorage{Time} >=0;  				# ... thermal energy storage
var onOffSOFC{j in TechsSOFC, Time, i in Units[j]} binary;			
                                                        # ... binary variables indicating what units of which SOFCs are turned on
var switch{j in TechsSOFC, Time, i in Units[j]} >= 0, <= 1;			
							# ... auxiliary variable for switching constraints

### OBJECTIVE ###

# ... minimize cost of acquisition and operation

minimize TotalCost: sum{j in Techs} CapCost[j] * numTech[j]     # ... 1st stage cost

	 	    + LifeExp/card(Time) * 
	
		    (sum{t in Time, j in TechsSOFC, i in Units[j]} (OpeMainCost[j] + (PriceGas[t] + EmiCarbGas)/RateElectrEff[j]) * power[j,t,i] * Beta^t                           # ... daily cost

		    + sum{t in Time} (PriceGas[t] + EmiCarbGas)/AveTherEff["Boiler"] * heatOut["Boiler",t] * Beta^t

		    + sum{t in Time} (PriceElectr[t] + EmiCarbPower) * powerUtility[t] * Beta^t

		    + sum{t in Time, j in TechsSOFC, i in Units[j]} SwitchCost[j] * switch[j,t,i] * Beta^t

		    + sum{p in Periods} min(card(Time)/30/24, 1.0) * PeakPrice[p] * maxPower[p] * Beta^EndPeriod[p]);         # ... monthly (=period) premium

### CONSTRAINTS ###
subject to

        # =================================  POWER  ==============================
	# ... power demand
	PowerDemandConstr{t in Time}:
          	sum{j in TechsSOFC, i in Units[j]} power[j,t,i] + powerUtility[t] - (powerInPlus[t] - powerInMinus[t]) >= PowerDemand[t];

	# ... on/off constraints on power SOFC and Chp SOFC
	MaxPower{t in Time, j in TechsSOFC, i in Units[j]}:
		MaxPowerRate[j] * onOffSOFC[j,t,i] >= power[j,t,i];
	
	MinPower{t in Time, j in TechsSOFC, i in Units[j]}: 
		MinPowerRate[j] * onOffSOFC[j,t,i] <= power[j,t,i];

	# ... only operate existing units (model number of units and onOffSOFC)
	ExistingUnits{j in TechsSOFC, t in Time}:
	         sum{i in Units[j]} onOffSOFC[j,t,i] <= numTech[j];

	# ... break symmetries between identical units
	SymmBreak{j in TechsSOFC, i in Units[j] diff {last(Units[j])}, t in Time}:
	       onOffSOFC[j,t,i] >= onOffSOFC[j,t,i+1];

	# ... auxiliary constraints for switching cost
	AuxConstrRight{j in TechsSOFC, i in Units[j], t in 1..card(Time)-1}:	
		switch[j,t,i] >= onOffSOFC[j,t+1,i] - onOffSOFC[j,t,i];

	AuxConstrLeft{j in TechsSOFC, i in Units[j], t in 1..card(Time)-1}:	
		switch[j,t,i] >= onOffSOFC[j,t,i] - onOffSOFC[j,t+1,i];

	# ... maximum utility 
	MaxUtility{p in Periods, t in StartPeriod[p]..EndPeriod[p]}: maxPower[p] >= powerUtility[t];

	# ... power storage from battery is upper bounded by battery capacity
	PowerStorageBound{t in Time}:
		powerStorage[t] <= BatteryCap * numTech["Battery"];

	# ... power output from battery is upper bounded by battery storage
	BatteryOutBound{t in Time}:
		powerInMinus[t] <= powerStorage[t];

	# ... energy storage
	EnergyStorage{t in Time diff {last(Time)}}:
              	powerStorage[t+1] = (1 - PowerLoss) * powerStorage[t] + (powerInPlus[t] - powerInMinus[t]);

	# ... boundary condition for battery
	BatteryBoundaryCond:
		powerStorage[1] = powerStorage[card(Time)];			
           
        # ... initial battery storage
	InitBattery:
		powerStorage[1] = 0;

        # =================================  HEAT ==============================

	# ... heat demand
	HeatDemandConstr{t in Time}:
		sum{j in TechsHeat} heatOut[j,t] >= HeatDemand[t];

	# ... heat input to storage bounded by heat generation from ChpSOFC
	HeatGen{t in Time diff {last(Time)}}:
		heatStorage[t+1] - (1 - HeatLoss) * heatStorage[t] + heatOut["Storage", t]
			 <= TherEff["ChpSOFC"]/RateElectrEff["ChpSOFC"] * sum{i in Units["ChpSOFC"]} power["ChpSOFC", t, i];

	# ... heat storage boundary and initial condition
	BoundaryCond:
		heatStorage[1] = heatStorage[card(Time)];

        InitHeatStorage:
                heatStorage[1] = 0;			

	# ... heat output from storage is upper bounded by storage itself
	HeatOutBound{t in Time}:
		heatOut["Storage",t] <= heatStorage[t];

	# ... heat storage is upper bounded the storage capacity	
	HeatStorageBound{t in Time}:
		heatStorage[t] <= TherCap * numTech["Storage"];

	# ... heat output from boiler is upper bounded by the boiler capacity
	HeatOutBoilerBound{t in Time}:
		heatOut["Boiler",t] <= BoilerCap * numTech["Boiler"];
