#########################################################################
# Generate snapshots for the MILP model of distributed generation (DG)  #
# AMPL coding by                                                        #
# Fu Lin                                                                #
# Argonne National Laboratory, 2014                                     #
#									#
# Description:                                                          #
# This script implements the Moving Horizon Method to generate profiles.#
#									#
# INPUTS:								#
# 1) "Building" -- the building type;					#
# 2) "HorizonDays" -- the number of days in the horizon;		#
# 3) "WindowLength" -- the length of the window;			#
#									#
# OUTPUTS:								#
# Snapshots saved in files *.sns for binary and continuous variables.	#
# Binary var -- OnOff;							#
# Continuous var -- HeatStorage, HeatOut, BattInputOutput, PowerStorage #
# PowerUtility, Power.							#
#########################################################################

### OPTIONS ###

#option Building School;
#Hospital School Retail Restaurant SuperMarket

#option HorizonDays 364;

option WindowLength 4;
option solver cplexamp;
option presolve_eps 1.e-12;
option cplex_options 'mipdisplay=2' 'mipinterval=1000' 'mipgap=1.0e-02' 'nodefile=2' 'time=10800' 'threads=1' 'prestats 1';
option DatFile ("data/" & $Building & "/" & $HorizonDays & "day.dat");
option log_file ("results/" & $Building & "/MovingHorizon_" & $HorizonDays  & "day.log");

# load model and data
model Full.mod;
data data/DG.dat;
data ($DatFile);

################# beginning of data preparation ################
set Days;
let Days := {1..(num($HorizonDays) + 1 - num($WindowLength))};
set LastDays;
let LastDays := {1..num($HorizonDays)} diff Days;

param DayCost{Days} default 0;
param MaxPowerDay{Days} default 0;
param _total_time default 0;

# save the first-stage solution for each day
param NumTechDays{Days,Techs};

# define an extra copy of time-dependent set and data	
set TimeHorizon;
let TimeHorizon := Time;
param HeatDemandHorizon {TimeHorizon};
param PowerDemandHorizon {TimeHorizon};
param PriceGasHorizon {TimeHorizon};
param PriceElectrHorizon {TimeHorizon};

let {t in Time} HeatDemandHorizon[t] := HeatDemand[t];
let {t in Time} PowerDemandHorizon[t] := PowerDemand[t];
let {t in Time} PriceGasHorizon[t] := PriceGas[t];
let {t in Time} PriceElectrHorizon[t] := PriceElectr[t];

# keep an extra copy for Periods
param PeakPriceHorizon{Periods};
param StartPeriodHorizon{Periods};
param EndPeriodHorizon{Periods};

let {p in Periods} PeakPriceHorizon[p] := PeakPrice[p];
let {p in Periods} StartPeriodHorizon[p] := StartPeriod[p];
let {p in Periods} EndPeriodHorizon[p] := EndPeriod[p];

# divide the window into the number of periods as in horizon
param NumPer := card(Periods);
let {p in 1..NumPer} EndPeriod[("period" & p)] := ceil(p*24*num($WindowLength)/NumPer);
let StartPeriod["period1"] := 1;
let {p in 2..NumPer} StartPeriod[("period" & p)] := 1 + EndPeriod[("period"&(p-1))];

# define the set of periods in the original MIP that the rolling window spans
set SpanPeriods default {};
set UsedPeriod default {};

################# end of data preparation ################

# hours in the rolling window
let Time := {1..24*num($WindowLength)};

# define initial conditions for the battery and water tank
param PowerStor default 0;
param HeatStor default 0;

# drop heat and battery initial condition constraints in the model
drop InitHeatStorage;
drop InitBattery;

# continuous variables
param PowerHorizon{j in TechsSOFC,TimeHorizon,i in Units[j]};
param PowerUtilityHorizon{TimeHorizon};
param PowerStorageHorizon{TimeHorizon};
param MaxPowerHorizon{Periods};
param BattIOHorizon{TimeHorizon};
param HeatOutHorizon{j in TechsHeat,TimeHorizon};
param HeatInHorizon{TimeHorizon};
param HeatStorageHorizon{TimeHorizon};

# binary variables
param OnOffSOFCHorizon{j in TechsSOFC,TimeHorizon,i in Units[j]};
param OnOffProfileHorizon{j in TechsSOFC,TimeHorizon};

# start solving for each day
for {d in Days}
    {

    # load data for second-stage MIP model
    let {t in Time} HeatDemand[t]  := HeatDemandHorizon[(d-1)*24+t];
    let {t in Time} PowerDemand[t] := PowerDemandHorizon[(d-1)*24+t];
    let {t in Time} PriceGas[t]    := PriceGasHorizon[(d-1)*24+t];
    let {t in Time} PriceElectr[t] := PriceElectrHorizon[(d-1)*24+t];

    # evaluate the period data
    let {p in Periods} PeakPrice[p] := PeakPriceHorizon[p];

    # load the initial condition for the battery and water tank
    fix powerStorage[1] := PowerStor;
    fix heatStorage[1] := HeatStor;

    # set the final condition for battery and water tank
    if d = card(Days) then {
        drop BoundaryCond;
	drop BatteryBoundaryCond;
        fix powerStorage[card(Time)] := 0;
    	fix heatStorage[card(Time)] := 0;}

    # start solving
    solve;

    display d;

    # set the initial condition for the battery and water tank
    let PowerStor := (1-PowerLoss)*powerStorage[24] + (powerInPlus[24] - powerInMinus[24]);
    let HeatStor  := (1-HeatLoss)*heatStorage[24] - heatOut["Storage", 24];

    let _total_time := _total_time + _solve_time;

    if d < card(Days) 
    then {

    # save power generated, purchased, and stored in the day
    let {t in 1..24, j in TechsSOFC, i in Units[j]} PowerHorizon[j,(d-1)*24+t,i] := power[j,t,i];
    let {t in 1..24} PowerUtilityHorizon[(d-1)*24+t] := powerUtility[t];
    let {t in 1..24} PowerStorageHorizon[(d-1)*24+t] := powerStorage[t];
    let {t in 1..24} BattIOHorizon[(d-1)*24+t] := powerInPlus[t] - powerInMinus[t];

    # save heat generated and stored in the day
    let {t in 1..24, j in TechsHeat} HeatOutHorizon[j,(d-1)*24+t] := heatOut[j,t];
    let {t in 1..24} HeatInHorizon[(d-1)*24+t] := heatIn[t];
    let {t in 1..24} HeatStorageHorizon[(d-1)*24+t] := heatStorage[t];
    
    # save on/off binary variables
    let {j in TechsSOFC,t in 1..24,i in Units[j]} OnOffSOFCHorizon[j,(d-1)*24+t,i] := onOffSOFC[j,t,i];
    let {j in TechsSOFC,t in 1..24} OnOffProfileHorizon[j,(d-1)*24+t] := sum{i in Units[j]} onOffSOFC[j,t,i];

    # save integer variables at the first-stage
    let {j in Techs} NumTechDays[d,j] := numTech[j];

    # save the maximum utility of the day
    let MaxPowerDay[d] := max {t in 1..24} powerUtility[t];
    }

    ############# save the data for the last few days at the end of the horizon #################
    if d = card(Days)
    then {

    # save power generated, purchased, and stored in the day
    let {t in Time, j in TechsSOFC, i in Units[j]} PowerHorizon[j,(d-1)*24+t,i] := power[j,t,i];
    let {t in Time} PowerUtilityHorizon[(d-1)*24+t] := powerUtility[t];
    let {t in Time} PowerStorageHorizon[(d-1)*24+t] := powerStorage[t];
    let {t in Time} BattIOHorizon[(d-1)*24+t] := powerInPlus[t] - powerInMinus[t];

    # save heat generated and stored in the day
    let {t in Time, j in TechsHeat} HeatOutHorizon[j,(d-1)*24+t] := heatOut[j,t];
    let {t in Time} HeatInHorizon[(d-1)*24+t] := heatIn[t];
    let {t in Time} HeatStorageHorizon[(d-1)*24+t] := heatStorage[t];
    
    # save on/off binary variables
    let {j in TechsSOFC,t in Time,i in Units[j]} OnOffSOFCHorizon[j,(d-1)*24+t,i] := onOffSOFC[j,t,i];
    let {j in TechsSOFC,t in Time} OnOffProfileHorizon[j,(d-1)*24+t] := sum{i in Units[j]} onOffSOFC[j,t,i];

    # save integer variables at the first-stage
    let {j in Techs} NumTechDays[d,j] := numTech[j];

    # save the maximum utility of the day
    let MaxPowerDay[d] := max {t in 1..24} powerUtility[t];
    }

}

################################ Save data ############################

option print_round 0;
    
# save binary variables into files
print {t in TimeHorizon}: {i in Units['ChpSOFC']} OnOffSOFCHorizon['ChpSOFC',t,i] 
		> ("snapshots/ChpSOFC/OnOff" & $Building  & $HorizonDays & "MovHorWinLen" & $WindowLength & "day.sns");
print {t in TimeHorizon}: {i in Units['PowerSOFC']} OnOffSOFCHorizon['PowerSOFC',t,i] 
		> ("snapshots/PowerSOFC/OnOff" & $Building  & $HorizonDays & "MovHorWinLen" & $WindowLength & "day.sns");
option print_round '';

option print_precision 0;

# save power variables into files
print {t in TimeHorizon}: {i in Units['ChpSOFC']} PowerHorizon['ChpSOFC',t,i] 
		> ("snapshots/ChpSOFC/Power" & $Building  & $HorizonDays & "MovHorWinLen" & $WindowLength & "day.sns");
print {t in TimeHorizon}: {i in Units['PowerSOFC']} PowerHorizon['PowerSOFC',t,i] 
		> ("snapshots/PowerSOFC/Power" & $Building  & $HorizonDays & "MovHorWinLen" & $WindowLength & "day.sns");
print {t in TimeHorizon}: PowerUtilityHorizon[t] 
		> ("snapshots/PowerUtility/" & $Building  & $HorizonDays & "MovHorWinLen" & $WindowLength & "day.sns");
print {t in TimeHorizon}: PowerStorageHorizon[t] 
		> ("snapshots/PowerStorage/" & $Building  & $HorizonDays & "MovHorWinLen" & $WindowLength & "day.sns");
print {t in TimeHorizon}: BattIOHorizon[t] 
		> ("snapshots/BattInputOutput/" & $Building  & $HorizonDays & "MovHorWinLen" & $WindowLength & "day.sns");

# save heat variables into files
print {t in TimeHorizon}: HeatOutHorizon['Boiler',t] 
		> ("snapshots/HeatOut/Boiler" & $Building  & $HorizonDays & "MovHorWinLen" & $WindowLength & "day.sns");
print {t in TimeHorizon}: HeatOutHorizon['Storage',t] 
		> ("snapshots/HeatOut/Storage" & $Building  & $HorizonDays & "MovHorWinLen" & $WindowLength & "day.sns");
print {t in TimeHorizon}: HeatStorageHorizon[t] 
		> ("snapshots/HeatStorage/" & $Building  & $HorizonDays & "MovHorWinLen" & $WindowLength & "day.sns");
