####################################################
# Group snapshots with respect to on/off profiles  #
# AMPL coding by                                   #
# Fu Lin                                           #
# Argonne National Laboratory, 2014                #
####################################################

#########################################################################
# Description:                                                          #
# This script removes redundant on/off profiles and group generation    #
# profiles associated with on/off profiles.                             #
# NOTE: This script should be used right after "ProfGen.ampl".		#
#									#
# INPUTS:								#
# 1) "Building" -- the building type;					#
# 2) "HorizonDays" -- the number of days in the horizon;		#
# 3) ".sns" -- snapshots for on/off and power generation profiles.      #
#									#
# Outputs:								#
# Snapshots saved in files *.sns for binary and continuous variables.	#
# Binary variables -> OnOff;						#
# Continuous variables -> HeatStorage, HeatOut, BattInputOutput		#
# PowerStorage, PowerUtility, Power.					#
#########################################################################

### OPTIONS ###

#option Building School;
#option HorizonDays 364;

#option NumDays 364;

option MovHor MovHorWinLen4;

# load model and data
model SemiCoarse.mod;
model CheckProf.mod;
data data/DG.dat;

### SETS ###
set ShuffledColumnIdx;				# ... set of column index

### PARAMETERS ###

# specify the number of days in the load profiles
param TotalNumDay;
param RawGenProf{j in TechsSOFC, 1..24*TotalNumDay, i in Units[j]};
param RawColumnPool{j in TechsSOFC, 1..24*TotalNumDay, i in Units[j]} default 2;  
						# ... default value is set to be different from 0 and 1

param ShuffledColumnPool{j in TechsSOFC, 1..24, 1..TotalNumDay*UpTech[j]} default 0;
						# ... define the matrix of size 24 by # of columns

param ShuffledGenProf{j in TechsSOFC, 1..24, 1..TotalNumDay*UpTech[j]} default 0;
						# ... define the matrxi of size 24 by # of production profiles

# start removing redundant columns
param ColDiff {1..24} default 0;     		# ... define the difference vector
param SumAbsColDiff default 0;       		# ... sum of absolute values of a vector (aka ell_1 norm)


param colJ;					# ...  parameters used in comparing columns
param colIflag default 0;

let TotalNumDay := num($NumDays);

# create column and generation pools from saved files
	read {t in 1..24*TotalNumDay}: {i in Units['PowerSOFC']} RawColumnPool["PowerSOFC",t,i] 
		< ("snapshots/PowerSOFC/OnOff" & $Building & $NumDays & $MovHor & "day.sns");  
	read {t in 1..24*TotalNumDay}: {i in Units['ChpSOFC']} RawColumnPool["ChpSOFC",t,i] 
		< ("snapshots/ChpSOFC/OnOff" & $Building & $NumDays & $MovHor & "day.sns");  
	read {t in 1..24*TotalNumDay}: {i in Units['PowerSOFC']} RawGenProf["PowerSOFC",t,i] 
		< ("snapshots/PowerSOFC/Power" & $Building & $NumDays & $MovHor & "day.sns");  
	read {t in 1..24*TotalNumDay}: {i in Units['ChpSOFC']} RawGenProf["ChpSOFC",t,i] 
		< ("snapshots/ChpSOFC/Power" & $Building & $NumDays & $MovHor & "day.sns");
	
let ShuffledColumnIdx := 1..TotalNumDay;

############################################################################################
# Sweeping through columns and profiles to get rid of redundant on/off profiles 
# and to group generations profiles with respect to on/off profiles.
# NOTES: this is done only once and the resulting non-redundant profiles are saved.
############################################################################################

for {j in TechsSOFC}{

    # put columns into matrix
    let {t in 1..24*TotalNumDay, i in Units[j]} 
     ShuffledColumnPool[j, ((t-1) mod 24)+1, floor((t-1)/24)*UpTech[j]+i] := RawColumnPool[j,t,i];

    # put generation profiles into matrix
    let {t in 1..24*TotalNumDay, i in Units[j]} 
     ShuffledGenProf[j, ((t-1) mod 24)+1, floor((t-1)/24)*UpTech[j]+i] := RawGenProf[j,t,i];

    # starting from column 1 and put it as a member of group 1
    let OnOffCol[j] := {1};
    let {h in 1..24} OnOffProf[h,j,1] := ShuffledColumnPool[j,h,1];

    let GenCol[j,1] := {1};
    let {h in 1..24} GenProf[h,j,1,1] := ShuffledGenProf[j,h,1];

    # for each column from 2..N
    # compare with existing groups of columns
    for {colI in 2..TotalNumDay*UpTech[j]}{

	    for {k in OnOffCol[j]}{

		    let colJ := first(GenCol[j,k]);      
		    # ... pick any one index in the set of GenCol. Here, we take the first member of that set.

		    let {t in 1..24} ColDiff[t] := ShuffledColumnPool[j,t,colI] - ShuffledColumnPool[j,t,colJ];
		    let SumAbsColDiff := sum {t in 1..24} abs(ColDiff[t]);

		    if SumAbsColDiff = 0             # ... if two columns are identical, put colI in the set of colJ
		    then {let GenCol[j,k] := GenCol[j,k] union {colI};
			  let {h in 1..24} GenProf[h,j,k,colI] := ShuffledGenProf[j,h,colI];
			  let colIflag := 1;}

	    }

	    if colIflag = 0       		# ... if colI does not belong to existing groups

	    then {let OnOffCol[j] := OnOffCol[j] union {card(OnOffCol[j])+1};      
						# ... then make colI a new group

		  let {h in 1..24} OnOffProf[h,j,card(OnOffCol[j])] := ShuffledColumnPool[j,h,colI];

		  let GenCol[j,card(OnOffCol[j])] := {colI};		      
						# ... and put index colI in the group

		  let {h in 1..24} GenProf[h,j,card(OnOffCol[j]),colI] := ShuffledGenProf[j,h,colI];}

	    else let colIflag := 0;		# ... otherwise, colI was already put in one of the groups in OnOffCol

    }

    printf "The number of groups for %s is %4i.\n", j, card(OnOffCol[j]);

}

printf "===========================================================================\n";
printf "Finished removing redundant on/off profiles and grouping generation profiles for\n";
printf "%s\n", ($Building & $NumDays & $MovHor & "day.sns");  
printf "===========================================================================\n";

##################### Save Non-redundant Snapshots #############################

# save on/off profiles
print {h in 1..24}: {k in OnOffCol['PowerSOFC']} OnOffProf[h,"PowerSOFC",k] 
	> ("snapshots/PowerSOFC/OnOff" & $Building & $NumDays & $MovHor & "day.nrs");  
print {h in 1..24}: {k in OnOffCol['ChpSOFC']} OnOffProf[h,"ChpSOFC",k] 
	> ("snapshots/ChpSOFC/OnOff" & $Building  & $NumDays & $MovHor & "day.nrs");  

# save generation profiles
for {j in TechsSOFC, k in OnOffCol[j]}{
	print {h in 1..24}: {l in GenCol[j,k]} GenProf[h,j,k,l] 
	> ("snapshots/" & j & "/" & $Building  & "OnOffProf" & k & "Power" & $NumDays & $MovHor & "day.nrs");}

# save the number of on/off profiles and the number generation profiles for each on/off profile
for {j in TechsSOFC}{
	print card(OnOffCol[j]), {k in OnOffCol[j]} card(GenCol[j,k])
	> ("snapshots/" & j & "/" & $Building  & "GenCol" & $NumDays & $MovHor & "day.set");}

printf "===========================================================================\n";
printf "Finished saving non-redundant profiles and the corresponding index sets\n";
printf "%s\n", ($Building & $NumDays & $MovHor & "day.sns");  
printf "===========================================================================\n";
