###############################################################
# Solve the coarse MILP model of distributed generation (DG)  #
# AMPL coding by                                              #
# Fu Lin                                                      #
# Argonne National Laboratory, 2014                           #
###############################################################

### OPTIONS ###

# specify options
#option Building Hospital;
#option HorizonDays 364;
#option NumDays 364;

option solver cplexamp;

option cplex_options 'mipdisplay=2' 'mipinterval=1000' 'mipgap=1.0e-02' 'nodefile=2' 'time=10800' 'threads=1' 'prestats 1';

#option presolve_eps 1.e-9;

option MovHor MovHorWinLen4;
option OutFile ("results/" & $Building & "/CoarseModel_" & $HorizonDays  &  "day"  &  $Method  & ".out");
option log_file ("results/" & $Building &"/CoarseModel_" & $HorizonDays  &  "day"  &  $Method  & ".log");
#option OutFile ("results/" & $Building & "/CoarseModel_" & $HorizonDays   & "day.out");
#option log_file ("results/" & $Building & "/CoarseModel_" & $HorizonDays  & "day.log");
option DatFile ("data/" & $Building & "/" & $HorizonDays & "day.dat");

### MODEL AND DATA ###

model Coarse.mod;
model CheckProf.mod;
data data/DG.dat;
data ($DatFile);

### SETS ###
set Hours := 1..24;					# ... number of hours in a day

### PARAMETERS ###
param total_solve_time default 0;
param con_eps default 1.e-11;				# ... epsilon test for constraint violations

param MaxIter := 100;					# ... maximum number of iterations for solving coarse model
param AddCon{1..MaxIter} default 0;			# ... number of violated constraints added to coarse model

param _ncons_csm;					# ... number of constraints for original coarse model

######################################################
	commands ProfLoad.ampl;
######################################################

######################################################

if $Method = "Sampling" 
   then	{commands ProfSelectSampling.ampl;}
   else if $Method = "Kmeans" 
        then {commands ProfSelectKmean.ampl;}
        else printf "Method for profile selection is either Sampling or Kmeans.";

######################################################


### drop hourly constraints for coarse model
drop HourlyHeatDemandConstr;
drop HourlyHeatInBound;
drop HourlyHeatOutBoilerBound;
drop HourlyHeatStorageBound;
drop HourlyMaxUtility;
drop HourlyOnUnitsBound;
drop HourlyPowerDemandConstr;
drop HourlyPowerStorageBound;
drop HourlySymmBreak; 

let _ncons_csm := _ncons;			# ... record the number of constraints for original coarse model

### solve the LP-relaxation to identify constraints  ###

# relax binary and integer variables
option relax_integrality 1;

commands SolveCoarseLP.ampl;

# restore binary and integer constraints
option relax_integrality 0;

### solve the model ###
solve; 
display  numTech, UpTech;
let AddCon[1] := _ncons - _ncons_csm;
let total_solve_time := total_solve_time + _solve_time;
printf '%13s%15s%15s%15s%15s%15s%15s\n', 'Year/Model', 'bVars', 'cVars', 'cons', 'Time', 'FirstStageSol', 'ObjVal' 
	> ($OutFile);

printf '%2i/Coar-Iter1%15i%15i%15i%15.2f (%2i,%2i,%2i,%2i,%2i) %15.2f\n', 
	num($HorizonDays)/364, _snbvars, _nvars-_snbvars, _ncons, _solve_time, 
	numTech['Battery'], numTech['Boiler'], numTech['ChpSOFC'], 
	numTech['PowerSOFC'], numTech['Storage'], TotalCost 
	> ($OutFile);

     for {r in 2..MaxIter}{

	   # restore hourly constraints that are violated 
	   restore {d in Days, h in Hours: 
		HourlyPowerDemandConstr[d,h].body <= HourlyPowerDemandConstr[d,h].lb - con_eps 
		or 
		HourlyPowerDemandConstr[d,h].body >= HourlyPowerDemandConstr[d,h].ub + con_eps} 
		HourlyPowerDemandConstr[d,h];

	   restore {d in Days, h in Hours: 
		HourlyPowerStorageBound[d,h].body <= HourlyPowerStorageBound[d,h].lb - con_eps
		or 
		HourlyPowerStorageBound[d,h].body >= HourlyPowerStorageBound[d,h].ub + con_eps} 
		HourlyPowerStorageBound[d,h];

	   restore {d in Days, h in Hours: 
		HourlyHeatDemandConstr[d,h].body <= HourlyHeatDemandConstr[d,h].lb - con_eps
		or
		HourlyHeatDemandConstr[d,h].body >= HourlyHeatDemandConstr[d,h].ub + con_eps} 
		HourlyHeatDemandConstr[d,h];

	   restore {d in Days, h in Hours: 
		HourlyHeatOutBoilerBound[d,h].body <= HourlyHeatOutBoilerBound[d,h].lb - con_eps
		or
		HourlyHeatOutBoilerBound[d,h].body >= HourlyHeatOutBoilerBound[d,h].ub + con_eps} 
		HourlyHeatOutBoilerBound[d,h];

	   restore {d in Days, h in Hours: 
		HourlyHeatStorageBound[d,h].body <= HourlyHeatStorageBound[d,h].lb - con_eps
		or
		HourlyHeatStorageBound[d,h].body >= HourlyHeatStorageBound[d,h].ub + con_eps} 
		HourlyHeatStorageBound[d,h];

	   restore {d in Days, h in 1..23: 
		HourlyHeatInBound[d,h].body <= HourlyHeatInBound[d,h].lb - con_eps
		or
		HourlyHeatInBound[d,h].body >= HourlyHeatInBound[d,h].ub + con_eps} 
		HourlyHeatInBound[d,h];

	   restore {d in Days, h in Hours, j in TechsSOFC: 
		HourlyOnUnitsBound[d,h,j].body <= HourlyOnUnitsBound[d,h,j].lb - con_eps
		or 
		HourlyOnUnitsBound[d,h,j].body >= HourlyOnUnitsBound[d,h,j].ub + con_eps} 
		HourlyOnUnitsBound[d,h,j];

	   restore {p in Periods, t in StartPeriod[p]..EndPeriod[p]: 
		HourlyMaxUtility[p,t].body <= HourlyMaxUtility[p,t].lb - con_eps
		or
		HourlyMaxUtility[p,t].body >= HourlyMaxUtility[p,t].ub + con_eps} 
		HourlyMaxUtility[p,t];        

	   restore {d in Days, h in 1..24, j in TechsSOFC, i in Units[j] diff {last(Units[j])}: 
		HourlySymmBreak[d,h,j,i].body <= HourlySymmBreak[d,h,j,i].lb - con_eps
		or
		HourlySymmBreak[d,h,j,i].body >= HourlySymmBreak[d,h,j,i].ub + con_eps} 
		HourlySymmBreak[d,h,j,i];

	   let AddCon[r] := _ncons - _ncons_csm;  

	   # if the number of added constraints are the same for the last two iterates
	   # then stop re-solving coarse model
	   if (r>=2) and (AddCon[r]=AddCon[r-1])
	   then {printf "No more constraints added. The number of solved coarse model is %2i. \n", r-1 > ($OutFile);
		break;}

	   # re-solve the coarse model with added constraints
	   solve; 
	   let total_solve_time := total_solve_time + _solve_time;
	   display numTech, UpTech;

	   printf '%2i/Coar-Iter%1i%15i%15i%15i%15.2f (%2i,%2i,%2i,%2i,%2i)%15.2f\n', 
		num($HorizonDays)/364, r, _snbvars, _nvars-_snbvars, _ncons, _solve_time, 
		numTech['Battery'], numTech['Boiler'], numTech['ChpSOFC'], numTech['PowerSOFC'], numTech['Storage'], TotalCost 
		> ($OutFile);
     }

### RECORD SOLUTION RESULTS ###
display total_solve_time;
display numTech, UpTech;
display _snbvars, _nvars-_snbvars, _ncons;
printf "Objective = %12.9f\n\n\n", TotalCost;

