#! /usr/bin/octave -q
# GenProf.m: generate demand profiles for distributed generation
# Simplifying Assumptions:
#    * month has exactly four weeks, i.e 28 days
#    * year has 13 months (i.e. 364 days) ... billing periods
#    * year has 8736 hours, 364 days, 52 weeks, 13 months
#
# Possible to change the parameterization of the demand and
# random, effects (e.g. introduce daily/weekly randomness, 
# rather than hourly random perturbation).
#
# Inputs are 
# (1) csv files (comma separated valued) generated from EnergyPlus. 
# (2) weight vector for weekly demand
# 
# Outputs are yearly demands:
# (1) TotalElectrDemand -> a single demand vector
# (2) TotalHeatingDemand -> a single demand vector
# (3) An AMPL data file that contains demand and price.
# 
# Sven Leyffer, Argonne National Laboratory, November 2014
# Modified by Fu Lin, Argonne National Laboratory, December 2014
###############################################################

###############################################################
### Define GLOBAL parameters 
PlotSwitch = 0;            % ... choose 1 for plots 0 without
SaveSwitch = 1;            % ... choose 1 to save daily and weekly data
HrsPerYear = 24*7*52;      % ... hours per year
SubSet = 1;		   % ... choose 1 for subset data
###############################################################

###############################################################
### Load simulation data from energy plus
                           % ... types of building include 
                           % ... SuperMarket,
                           % ... FullServiceRestaurant, Hospital, 
                           % ... SecondarySchool, StandAloneRetail
close all;
#typecode = 1;
switch typecode
       case 1
	 type = 'School';         
       case 2
	 type = 'Restaurant';         
       case 3
	 type = 'SuperMarket';
       case 4
	 type = 'Hospital';
       case 5
	 type = 'Retail';
endswitch

#yearcode = 1;
switch yearcode
	case 1
	YearNum = 'One';
	case 2
	YearNum = 'Two';
	case 4
	YearNum = 'Four';
	case 6
	YearNum = 'Six';
	case 8
       	YearNum = 'Eight';
	case 10
       	YearNum = 'Ten';
	#case 20
	#YearNum = 'Twenty';
endswitch
              

#csvname = ['CSV_files/RefBldg',type,'New2004_ChicagoMeter.csv'];

csvname = ['CSV_files/',type,'.csv'];
SimData = csvread(csvname,1,1);    
                           % ... read comma-separated value files 
                           % ... parameters (1,1) means start reading data from (1,1) entry
                           % ... to exclude headers and the first column in the csv file

% extract aggregated electricity and heating demands for winter and summer days
ElectrWinter = SimData(1:24,1);
ElectrSummer = SimData(25:48,1);
                           % ... column 1 in SimData is the aggregated Electricity demand

HeatingGasWinter = SimData(1:24,8);
HeatingGasSummer = SimData(25:48,8);
                           % ... column 8 in SimData is the aggregated HeatingGas demand

if (PlotSwitch)
    figure, 
    subplot(2,1,1); plot(1:24,ElectrWinter); 
    title(['Electricity for ', type, ' in a winter day']);
    subplot(2,1,2); plot(1:24,ElectrSummer); 
    title(['Electricity for ', type, ' in a summer day']);
    figure, 
    subplot(2,1,1); plot(1:24,HeatingGasWinter); 
    title(['Heating Gas for ', type, ' in a winter day']);
    subplot(2,1,2); plot(1:24,HeatingGasSummer); 
    title(['Heating Gas for ', type, ' in a summer day']);
endif



###############################################################
### Generate weekly data based on winter and summer days

quarterday = 0.25;
halfday = 0.5;
fullday = 1;

% weight vector for building type
switch type 
       case 'SuperMarket'
       WeeklyWeight = [halfday*ones(5,1);fullday;fullday];

       case 'Restaurant'
       WeeklyWeight = [quarterday; quarterday; halfday; halfday; fullday; fullday; halfday];

       case 'Hospital' 
       WeeklyWeight = fullday*ones(7,1);

       case 'School'
       WeeklyWeight = [ones(5,1); halfday; quarterday];

       case 'Retail'
       WeeklyWeight = [quarterday; halfday; halfday; fullday; fullday; quarterday; quarterday];
endswitch 

% electricity demand
WeekElectrWinter = kron(WeeklyWeight,ElectrWinter);
WeekElectrSummer = kron(WeeklyWeight,ElectrSummer);

% heating gas demand
WeekHeatingGasWinter = kron(WeeklyWeight, HeatingGasWinter);
WeekHeatingGasSummer = kron(WeeklyWeight, HeatingGasSummer);

if (PlotSwitch)
  figure; 
  subplot(2,1,1); plot(1:1:24*7, WeekElectrWinter); 
  title(['Electricity demand for ', type, ' in a winter week']);
  subplot(2,1,2); plot(1:1:24*7, WeekElectrSummer); 
  title(['Electricity demand for ', type, ' in a summer week']);
  figure; 
  subplot(2,1,1); plot(1:1:24*7, WeekHeatingGasWinter); 
  title(['HeatingGas demand for ', type, ' in a winter week']);
  subplot(2,1,2); plot(1:1:24*7, WeekHeatingGasSummer); 
  title(['HeatingGas demand for ', type, ' in a summer week']);
endif

###############################################################
### Generate yearly data based on winter and summer weeks

# piecewise linear interpolation between winter and summer weeks
IntWeek = 1:1:52;
IntProf = [ones(12,1); linspace(1,0,12)'; zeros(12,1); linspace(0,1,8)'; ones(8,1)];

if (PlotSwitch)
  figure; plot(IntWeek, IntProf); 
  title('Piece-wise linear interpolation between winter and summer days');
endif

# yearly data generated by interpolation 
YearElectr = kron(IntProf,WeekElectrWinter) + kron(1-IntProf,WeekElectrSummer);
YearHeatingGas = kron(IntProf,WeekHeatingGasWinter) + kron(1-IntProf,WeekHeatingGasSummer);

if (PlotSwitch)
  figure; plot(YearElectr);
  title('Yearly Electricity demand');
  figure; plot(YearHeatingGas);
  title('Yearly HeatingGas demand');
endif

################################################################
### Save data to ampl readable format
# convert joules to kWh, recall that 1 kWh = 3.6 x 10^6 Joules.

# heat and power demands are in kWh units
ratio = 3.6e+6;
HeatDemand = YearHeatingGas/ratio;
PowerDemand = YearElectr/ratio;

# We follow NYC tarrif Table 10 Energy Prices effective April 2007
# published in "Effect of heat and electricity storage and reliability on microgrid viability:
# A study of commericial buildings in California and New York states" by
# M. Stadler et al. 2009 
# at Berkeley National Laboratory

# Summer (June-Sep) Electricity 0.120 $/kWh, Gas 0.049 $/kWh, demand charge 14.2 $/kW
# Winter (Oct.-May) Electricity 0.120 $/kWh, Gas 0.049 $/kWh, demand charge 11.36 $/kW

PriceElectr = 0.12*ones(HrsPerYear,1);
PriceGas = 0.049*ones(HrsPerYear,1);
DemandSumm = 14.2;
DemandWint = 11.36;

# add random noise
NSR = 0.02;             % ... noise signal ratio
PowerDemand = PowerDemand + NSR*randn(size(PowerDemand)) .* abs(PowerDemand);
HeatDemand  = HeatDemand  + NSR*randn(size(HeatDemand))  .* abs(HeatDemand);

DailyData = [[1:1:24]', ...
             PowerDemand(1:24), PowerDemand(4201:4224), ...
	     HeatDemand(1:24), HeatDemand(4201:4224)];

WeeklyData = [[1:1:168]', ...
             PowerDemand(1:168), PowerDemand(4201:4368), ...
	     HeatDemand(1:168), HeatDemand(4201:4368)];

TotalData = [(1:1:yearcode*HrsPerYear)',...
		kron(ones(yearcode,1),HeatDemand) + NSR*randn(yearcode*length(HeatDemand),1) ...
						    .*abs(kron(ones(yearcode,1),HeatDemand)) ...
		kron(ones(yearcode,1),PowerDemand) + NSR*randn(yearcode*length(PowerDemand),1) ...
						    .*abs(kron(ones(yearcode,1),PowerDemand)), ...
		kron(ones(yearcode,1),PriceGas), ...
		kron(ones(yearcode,1),PriceElectr)];

if (SaveSwitch)

  ### save daily data in a table ### 
  fileName = ['data/', type, '/', 'Day.dat'];
  [fileID, MSG] = fopen(fileName,'w');
  fprintf(fileID, '%6s%20s%20s%20s%20s\n', 'Time', 'ElectrWinter', 'ElectrSummer', 'HeatingGasWinter', 'HeatingGasSummer');
  fprintf(fileID, '%6i%20.2f%20.2f%20.2f%20.2f\n', DailyData');
  fclose(fileID);

  ### save weekly data in a table ### 
  fileName = ['data/', type, '/', 'Week.dat'];
  [fileID, MSG] = fopen(fileName,'w');
  fprintf(fileID, '%6s%20s%20s%20s%20s\n', 'Time', 'ElectrWinter', 'ElectrSummer', 'HeatingGasWinter', 'HeatingGasSummer');
  fprintf(fileID, '%6i%20.2f%20.2f%20.2f%20.2f\n', WeeklyData');
  fclose(fileID);

  ### save year data in ampl data format ###
  fileName = ['data/', type, '/', num2str(yearcode*364), 'day.dat'];
  [fileID, MSG] = fopen(fileName,'w');

  # print out the demand and price data set to file
  fprintf(fileID, '%6s\n', 'param:');
  fprintf(fileID, '%6s%15s%15s%15s%20s\n', 'Time:', 'HeatDemand', 'PowerDemand',  'PriceGas', 'PriceElectr :=');
  fprintf(fileID, '%6i%15.2f%15.2f%15.2f%20.2f\n', TotalData');
  fprintf(fileID, '%1s\n\n\n',';');

  # print out the period data set to file
  startperiod = zeros(13*yearcode,1); 
  startperiod(1) = 1;
  endperiod = 24*28*(1:1:13*yearcode)';
  startperiod(2:end) = endperiod(1:end-1)+1;
  DemandCharge = kron(ones(yearcode,1), [DemandWint*ones(6,1);DemandSumm*ones(4,1);DemandWint*ones(3,1)]);
  PeriodData = [DemandCharge,startperiod,endperiod];

  fprintf(fileID, '%6s\n', 'param:');
  fprintf(fileID, '%10s%15s%15s%15s\n','Periods:','PeakPrice','StartPeriod','EndPeriod :=');

  for i = 1:13*yearcode
    fprintf(fileID, '%10s%15.2f%15i%15i\n', ['period',num2str(i)], PeriodData(i,:)');
  endfor

  fprintf(fileID, '%1s\n\n\n',';');
  fclose(fileID);
endif

if (SubSet)

  for years = [1,2,4,6,8,10]

	### save year data in ampl data format ###
  	fileName = ['data/', type, '/', num2str(years*364), 'day.dat'];
	[fileID, MSG] = fopen(fileName,'w');

  	# print out the demand and price data set to file
  	fprintf(fileID, '%6s\n', 'param:');
	fprintf(fileID, '%6s%15s%15s%15s%20s\n', 'Time:', 'HeatDemand', 'PowerDemand',  'PriceGas', 'PriceElectr :=');
	fprintf(fileID, '%6i%15.2f%15.2f%15.2f%20.2f\n', TotalData(1:years*HrsPerYear,:)');
	fprintf(fileID, '%1s\n\n\n',';');

	# print out the period data set to file
	startperiod = zeros(13*years,1); 
	startperiod(1) = 1;
	endperiod = 24*28*(1:1:13*years)';
	startperiod(2:end) = endperiod(1:end-1)+1;
	DemandCharge = kron(ones(years,1), [DemandWint*ones(6,1);DemandSumm*ones(4,1);DemandWint*ones(3,1)]);
	PeriodData = [DemandCharge,startperiod,endperiod];

	fprintf(fileID, '%6s\n', 'param:');
	fprintf(fileID, '%10s%15s%15s%15s\n','Periods:','PeakPrice','StartPeriod','EndPeriod :=');

	for i = 1:13*years
	  fprintf(fileID, '%10s%15.2f%15i%15i\n', ['period',num2str(i)], PeriodData(i,:)');
	endfor

	fprintf(fileID, '%1s\n\n\n',';');
	fclose(fileID);

  endfor

endif
